# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .text_part_type import TextPartType


class TextPart(UniversalBaseModel):
    """
    Text content part.
    """

    text: str = pydantic.Field()
    """
    The text content
    """

    type: typing.Optional[TextPartType] = pydantic.Field(default=None)
    """
    Part type discriminator
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
