# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel


class AccessGroup(UniversalBaseModel):
    created_at: typing.Optional[dt.datetime] = pydantic.Field(default=None)
    """
    Creation timestamp.
    """

    created_by: str = pydantic.Field()
    """
    member_id of the creator.
    """

    description: typing.Optional[str] = pydantic.Field(default=None)
    """
    Optional group description.
    """

    id: str = pydantic.Field()
    """
    Access group id.
    """

    name: str = pydantic.Field()
    """
    Group name.
    """

    owner_org_id: str = pydantic.Field()
    """
    Owner organization id.
    """

    updated_at: typing.Optional[dt.datetime] = pydantic.Field(default=None)
    """
    Last update timestamp.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
