# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .schedule_list_item import ScheduleListItem


class ScheduleListResponse(UniversalBaseModel):
    """
    Response model for listing schedules
    """

    schedules: typing.List[ScheduleListItem] = pydantic.Field()
    """
    List of schedules
    """

    total: int = pydantic.Field()
    """
    Total number of schedules
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
