# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .version_pod_log_entry import VersionPodLogEntry


class VersionPodLogsResponse(UniversalBaseModel):
    """
    Paginated pod logs for a deployed version.
    """

    has_more: bool = pydantic.Field()
    """
    Whether more matching logs exist beyond this page
    """

    logs: typing.Optional[typing.List[VersionPodLogEntry]] = None
    total: int = pydantic.Field()
    """
    Total number of matching logs
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
